/* 
   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* BOOTP standard vendor field size */
#define BOOTP_VENDOR    64

#if 1
/* DHCP extended vendor field size */
#define BOOTP_DHCPVEND  1024
#else
/* DHCP standard vendor field size */
#define BOOTP_DHCPVEND  312
#endif

#define	IPPORT_BOOTPS		67
#define	IPPORT_BOOTPC		68

#define BOOTREQUEST		1
#define BOOTREPLY		2

#define BP_CHADDR_LEN	 	16
#define BP_SNAME_LEN	 	64
#define BP_FILE_LEN		128
#define BP_VEND_LEN	 	64
#define BP_MINPKTSZ		300	/* to check sizeof(struct bootp) */

/*
 *	We use unsigned long instead of in_addr for bp_?iaddr because
 *	we don't want to drag in netinet/in.h
 */

struct bootp_t
{
	unsigned char	bp_op;			/* packet opcode type */
	unsigned char	bp_htype;		/* hardware addr type */
	unsigned char	bp_hlen;		/* hardware addr length */
	unsigned char	bp_hops;		/* gateway hops */
	unsigned long	bp_xid;			/* transaction ID */
	u_int16_t		bp_secs;		/* seconds since boot began */
	u_int16_t		bp_flags;		/* RFC1532 broadcast, etc. */
	u_int32_t		bp_ciaddr;		/* client IP address */
	u_int32_t		bp_yiaddr;		/* 'your' IP address */
	u_int32_t		bp_siaddr;		/* server IP address */
	u_int32_t		bp_giaddr;		/* gateway IP address */
	unsigned char	bp_chaddr[BP_CHADDR_LEN];/* client hardware address */
	char		bp_sname[BP_SNAME_LEN];	/* server host name */
	char		bp_file[BP_FILE_LEN];	/* boot file name */
	unsigned char	bp_vend[BP_VEND_LEN];	/* vendor-specific area */
	/* note that bp_vend can be longer, extending to end of packet. */
};

/* BOOTstrap Protocol (BOOTP) header */
typedef struct bootph {
    u_int8_t opcode,                    /* operation code */
        hardware,                       /* hardware type */
        hardlen,                        /* length of hardware address */
        gatehops;                       /* gateways hops */
    u_int32_t ident;                    /* transaction identification */
    u_int16_t seconds,                  /* seconds elapsed since boot began */
        flags;                          /* flags */
    ip4_t cip,                          /* client IP address */
        yip,                            /* your IP address */
        sip,                            /* server IP address */
        gip;                            /* gateway IP address */
    mac_addr_t caddr,                   /* client hardware address */
        sname[64],                      /* server name */
        bootfile[128];                  /* bootfile name */
    union {
        u_int8_t d[BOOTP_DHCPVEND];     /* vendor-specific stuff */
        struct {
            u_int8_t   magic[4];         /* magic number */
            u_int32_t flags;            /* flags/opcodes etc */
            u_int8_t  pad[56];          /* padding chars */
        } v;
    } vendor;
} bootplayer_t;

#define VM_RFC1048      0x63825363L     /* RFC1048 magic number */
#define BOOTP_SPORT     67              /* BOOTP server port */
#define BOOTP_CPORT     68              /* .. client port */

#define BOOTP_REQ       1               /* BOOTP request */
#define BOOTP_REP       2               /* .. reply */

/* BOOTP flags field */
#define BOOTP_BCAST     0x8000          /* BOOTP broadcast flag */
#define BOOTP_FLAGS     BOOTP_BCAST     /* .. for FDDI address transl. */

/*
 * Tag values used to specify what information is being supplied in
 * the vendor (options) data area of the packet.
 */
/* RFC 1048 */
#define TAG_END			((unsigned char) 255)
#define TAG_PAD			((unsigned char)   0)
#define TAG_SUBNET_MASK		((unsigned char)   1)
#define TAG_TIME_OFFSET		((unsigned char)   2)
#define TAG_GATEWAY		((unsigned char)   3)
#define TAG_TIME_SERVER		((unsigned char)   4)
#define TAG_NAME_SERVER		((unsigned char)   5)
#define TAG_DOMAIN_SERVER	((unsigned char)   6)
#define TAG_LOG_SERVER		((unsigned char)   7)
#define TAG_COOKIE_SERVER	((unsigned char)   8)
#define TAG_LPR_SERVER		((unsigned char)   9)
#define TAG_IMPRESS_SERVER	((unsigned char)  10)
#define TAG_RLP_SERVER		((unsigned char)  11)
#define TAG_HOST_NAME		((unsigned char)  12)
#define TAG_BOOT_SIZE		((unsigned char)  13)
/* RFC 1395 */
#define TAG_DUMP_FILE		((unsigned char)  14)
#define TAG_DOMAIN_NAME		((unsigned char)  15)
#define TAG_SWAP_SERVER		((unsigned char)  16)
#define TAG_ROOT_PATH		((unsigned char)  17)
/* RFC 1497 */
#define TAG_EXTEN_FILE		((unsigned char)  18)
/* RFC 1533 */
#define TAG_NIS_DOMAIN		((unsigned char)  40)
#define TAG_NIS_SERVER		((unsigned char)  41)
#define TAG_NTP_SERVER		((unsigned char)  42)
/* DHCP maximum message size. */
#define TAG_MAX_MSGSZ		((unsigned char)  57)

extern int fill_bootp_request(struct bootp_t *bootp_packet,
	char *macaddr, long xid);
